from mage_ai.data_preparation.repo_manager import get_repo_path
from mage_ai.orchestration.db.models.schedules import PipelineSchedule
from mage_ai.tests.api.operations.test_base import BaseApiTestCase
from mage_ai.tests.factory import create_pipeline, create_user


class PipelineScheduleOperationTests(BaseApiTestCase):
    model_class = PipelineSchedule

    @property
    def model_class_name(self) -> str:
        return 'pipeline_schedule'

    @classmethod
    def setUpClass(self):
        super().setUpClass()
        self.pipeline = create_pipeline('test pipeline', self.repo_path)

    async def test_execute_create_schedule(self):
        email = self.faker.email()
        user = create_user(email=email, roles=1)
        operation = self.build_create_operation(
            dict(
                name='test_schedule',
                schedule_interval='@daily',
                start_time='2023-01-01T00:00:00',
                status='inactive',
                variables=[],
            ),
            resource_parent='pipeline',
            resource_parent_id=self.pipeline.uuid,
            user=user,
        )
        response = await operation.execute()

        self.assertEqual(response['pipeline_schedule']['name'], 'test_schedule')
        self.assertEqual(response['pipeline_schedule']['repo_path'], get_repo_path())

    async def test_execute_list(self):
        email = self.faker.email()
        user = create_user(email=email, roles=1)

        await self.base_test_execute_list(
            [
                dict(
                    name='test_schedule',
                    schedule_interval='@daily',
                    start_time='2023-01-01T00:00:00',
                    status='inactive',
                    variables=[],
                    repo_path=None,
                ),
                dict(
                    name='test_schedule_2',
                    schedule_interval='@daily',
                    start_time='2023-01-01T00:00:00',
                    status='inactive',
                    variables=[],
                ),
            ],
            [
                'id',
            ],
            resource_parent='pipeline',
            resource_parent_id=self.pipeline.uuid,
            user=user,
        )



SQL_SOURCES = [
    dict(name='BigQuery'),
    dict(
        name='Microsoft SQL Server',
        uuid='mssql',
    ),
    dict(name='MongoDB'),
    dict(name='MySQL'),
    dict(name='OracleDB'),
    dict(name='PostgreSQL'),
    dict(name='Redshift'),
    dict(name='Snowflake'),
]

SOURCES = sorted([
    dict(name='Amazon S3'),
    dict(name='Amplitude'),
    dict(name='Api'),
    dict(name='Azure Blob Storage'),
    dict(name='Chargebee'),
    dict(name='Commercetools'),
    dict(name='Couchbase'),
    dict(name='Datadog'),
    dict(name='DynamoDB'),
    dict(name='Facebook Ads'),
    dict(name='Freshdesk'),
    dict(name='Front'),
    dict(name='Google Ads'),
    dict(name='Google Analytics'),
    dict(name='Google Search Console'),
    dict(name='Google Sheets'),
    dict(name='HubSpot'),
    dict(name='Intercom'),
    dict(name='LinkedIn Ads'),
    dict(name='Monday'),
    dict(name='Outreach'),
    dict(name='Paystack'),
    dict(name='Pipedrive'),
    dict(name='Postmark'),
    dict(name='Salesforce'),
    dict(name='Stripe'),
    dict(name='Twitter Ads'),
    dict(name='Zendesk'),
] + SQL_SOURCES, key=lambda x: x['name'])



from enum import Enum

DEFAULT_BATCH_SIZE = 100


class SourceType(str, Enum):
    AMAZON_SQS = 'amazon_sqs'
    AZURE_EVENT_HUB = 'azure_event_hub'
    GOOGLE_CLOUD_PUBSUB = 'google_cloud_pubsub'
    KAFKA = 'kafka'
    KINESIS = 'kinesis'
    RABBITMQ = 'rabbitmq'


class SinkType(str, Enum):
    AMAZON_S3 = 'amazon_s3'
    DUMMY = 'dummy'
    KAFKA = 'kafka'
    KINESIS = 'kinesis'
    MONGODB = 'mongodb'
    OPENSEARCH = 'opensearch'



"""Add executor_type to pipeline_run.

Revision ID: 66e67039b8a2
Revises: 21e31d66ccea
Create Date: 2023-05-12 11:20:50.288324

"""
from alembic import op
from mage_ai.data_preparation.models.constants import ExecutorType
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '66e67039b8a2'
down_revision = '21e31d66ccea'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # bind = op.get_bind()
    executor_type_enum = sa.Enum(ExecutorType, name='executortype')
    executor_type_enum.create(op.get_bind(), checkfirst=True)
    with op.batch_alter_table('pipeline_run', schema=None) as batch_op:
        batch_op.add_column(sa.Column('executor_type', executor_type_enum, nullable=True))


def downgrade() -> None:
    with op.batch_alter_table('pipeline_run', schema=None) as batch_op:
        batch_op.drop_column('executor_type')
    executor_type_enum = sa.Enum(ExecutorType, name='executortype')
    executor_type_enum.drop(op.get_bind())



from dataclasses import dataclass
from mage_ai.shared.config import BaseConfig


@dataclass
class StitchConfig(BaseConfig):
    """
    How to obtain an API access token:
    https://www.stitchdata.com/docs/developers/stitch-connect/api#obtain-access-token
    """
    access_token: str



from mage_ai.data_preparation.repo_manager import get_repo_path
from mage_ai.io.config import ConfigFileLoader
from mage_ai.io.redshift import Redshift
from os import path

if 'sensor' not in globals():
    from mage_ai.data_preparation.decorators import sensor


@sensor
def query_redshift_and_check_condition(*args, **kwargs) -> bool:
    """
    Template code for checking the results of a Redshift query.
    Specify your configuration settings in 'io_config.yaml'.

    Return: True if the sensor should complete, False if it should
    keep waiting
    """

    config_path = path.join(get_repo_path(), 'io_config.yaml')
    config_profile = 'default'

    query = 'Your Redshift query'  # Specify your SQL query here

    with Redshift.with_config(
            ConfigFileLoader(config_path, config_profile)) as loader:
        df = loader.load(query)

        # Add your checks here
        if df.empty:
            return False

    return True



from mage_ai.services.airbyte.client import AirbyteClient
from mage_ai.services.airbyte.config import AirbyteConfig
from mage_ai.services.airbyte.constants import (
    CONNECTION_STATUS_ACTIVE,
    CONNECTION_STATUS_DEPRECATED,
    CONNECTION_STATUS_INACTIVE,
    JOB_STATUS_FAILED,
    JOB_STATUS_PENDING,
    JOB_STATUS_SUCCEEDED,
)
from mage_ai.services.airbyte.exceptions import (
    ConnectionDeprecated,
    ConnectionInactive,
    SyncJobFailed,
)
from mage_ai.services.airbyte.server import AirbyteServer
from mage_ai.utils.logger import Logger
from time import sleep
from typing import Dict


class Airbyte():
    def __init__(
        self,
        logger: Logger = None,
        **kwargs,
    ):
        self.logger = logger or Logger(caller='Airbyte', log_to_stdout=True)
        self.server = AirbyteServer(AirbyteConfig(**kwargs))

    def run_sync(self, connection_id: str, poll_interval: int = 10, timeout: int = 5) -> Dict:
        self.logger.info('Run sync started.')

        with self.server.get_client(logger=self.logger, timeout=timeout) as client:
            connection_status = client.get_connection_status(connection_id)

            if CONNECTION_STATUS_ACTIVE == connection_status:
                job = self.__run_sync(client, connection_id, poll_interval=poll_interval)

                return dict(
                    connection_id=connection_id,
                    connection_status=connection_status,
                    job=job,
                )
            elif CONNECTION_STATUS_INACTIVE == connection_status:
                msg = f'Connection: {connection_id} is inactive.'
                self.logger.error(msg)
                raise ConnectionInactive(msg)
            elif CONNECTION_STATUS_DEPRECATED == connection_status:
                msg = f'Connection {connection_id} is deprecated.'
                self.logger.error(msg)
                raise ConnectionDeprecated(msg)

    def __run_sync(
        self,
        client: AirbyteClient,
        connection_id: str,
        poll_interval: int = 10,
    ) -> Dict:
        """
        Arguments:
            poll_interval: How many seconds in between each job status check.
        """

        job = client.run_sync(connection_id)
        job_id = job['id']

        status = JOB_STATUS_PENDING
        while status not in [JOB_STATUS_FAILED, JOB_STATUS_SUCCEEDED]:
            job = client.get_job_status(job_id)
            status = job['status']

            if JOB_STATUS_SUCCEEDED == status:
                self.logger.info(f'Job {job_id} succeeded.')
            elif JOB_STATUS_FAILED == status:
                msg = f'Job {job_id} failed.'
                self.logger.error(msg)
                raise SyncJobFailed(msg)
            else:
                self.logger.info(f'Job {job_id} status: {status}.')
                sleep(poll_interval)

        return job



{% extends "transformers/transformer_actions/action.jinja" %}
{% block action %}
    """
    Execute Transformer Action: ActionType.DROP_DUPLICATE

    Docs: https://docs.mage.ai/guides/transformer-blocks#drop-duplicates
    """
    action = build_transformer_action(
        df,
        action_type=ActionType.DROP_DUPLICATE,
        arguments=df.columns,  # Specify column names to use when comparing duplicates
        axis=Axis.ROW,
        options={'keep': 'first'},  # Specify whether to keep 'first' or 'last' duplicate
    )
{% endblock %}



from mage_ai.orchestration.pipeline_scheduler import schedule_with_event


class EventTrigger:
    def run(self, event) -> None:
        print(f'Trigger by event: {event}')
        schedule_with_event(event)
